@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Gabriele Greco <gabrielegreco@gmail.com>
  *
@@ -45,6 +45,7 @@ private:
     static const size_t _buffer_size;   // Size of each audio buffer
 
     // OpenAL things
+    std::vector<std::string> _devices;  // List of known OpenAL devices
     bool _initialized;                  // Was this initialized?
     ALCdevice *_device;                 // Audio device        
     ALCcontext *_context;               // Audio context associated with device
@@ -63,6 +64,9 @@ private:
     int64_t _ext_timer_at_last_timestamp;
     int64_t _last_reported_timestamp;
 
+    // Parameters
+    parameters _params;
+
     // Get an OpenAL source format for the audio data in blob (or throw an exception)
     ALenum get_al_format(const audio_blob &blob);
 
@@ -70,11 +74,20 @@ public:
     audio_output();
     ~audio_output();
     
-    /* Initialize the audio device for output. Throw an exception if this fails. */
-    void init();
+    /* How many OpenAL devices are available? */
+    int devices() const;
+    /* Return the name of OpenAL device i. */
+    const std::string &device_name(int i) const;
+
+    /* Initialize the audio device i for output. If i is -1, the default device
+     * will be used. Throw an exception if this fails. */
+    void init(int i = -1);
     /* Deinitialize the audio device. */
     void deinit();
 
+    /* Set parameters. */
+    void set_parameters(const parameters &params);
+
     /* To play audio, do the following:
      * - First, call required_initial_data_size() to find out the initial amount
      *   of audio data that is required.
