@@ -1,7 +1,5 @@
 /*
- * This file is part of bino, a 3D video player.
- *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011
  * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +19,9 @@
 #include "config.h"
 
 #include <cerrno>
-#ifndef HAVE_CLOCK_GETTIME
+#ifdef HAVE_CLOCK_GETTIME
+# include <time.h>
+#else
 # include <sys/time.h>
 #endif
 
@@ -38,25 +38,20 @@ int64_t timer::get_microseconds(timer::type t)
 
     struct timespec time;
     int r;
-    if (t == realtime)
-    {
+    if (t == realtime) {
         r = clock_gettime(CLOCK_REALTIME, &time);
-    }
-    else if (t == monotonic)
-    {
+    } else if (t == monotonic) {
         r = clock_gettime(CLOCK_MONOTONIC, &time);
-    }
-    else if (t == process_cpu)
-    {
-#ifdef CLOCK_PROCESS_CPUTIME_ID
+    } else if (t == process_cpu) {
+#if defined(CLOCK_PROCESS_CPUTIME_ID)
         r = clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &time);
+#elif defined(CLOCK_PROF)
+        r = clock_gettime(CLOCK_PROF, &time);
 #else
         r = -1;
         errno = ENOSYS;
 #endif
-    }
-    else // t == thread_cpu
-    {
+    } else /* t == thread_cpu */ {
 #ifdef CLOCK_THREAD_CPUTIME_ID
         r = clock_gettime(CLOCK_THREAD_CPUTIME_ID, &time);
 #else
@@ -65,33 +60,24 @@ int64_t timer::get_microseconds(timer::type t)
 #endif
     }
     if (r != 0)
-    {
         throw exc(_("Cannot get time."), errno);
-    }
     return static_cast<int64_t>(time.tv_sec) * 1000000 + time.tv_nsec / 1000;
 
 #else
 
-    if (t == realtime || t == monotonic)
-    {
+    if (t == realtime || t == monotonic) {
         struct timeval tv;
         int r = gettimeofday(&tv, NULL);
         if (r != 0)
-        {
             throw exc(_("Cannot get time."), errno);
-        }
         return static_cast<int64_t>(tv.tv_sec) * 1000000 + tv.tv_usec;
-    }
-    else if (t == process_cpu)
-    {
+    } else if (t == process_cpu) {
         // In W32, clock() starts with zero on program start, so we do not need
         // to subtract a start value.
         // XXX: Is this also true on Mac OS X?
         clock_t c = clock();
         return (c * static_cast<int64_t>(1000000) / CLOCKS_PER_SEC);
-    }
-    else
-    {
+    } else {
         throw exc(_("Cannot get time."), ENOSYS);
     }
 
