@@ -1,7 +1,5 @@
 /*
- * This file is part of bino, a 3D video player.
- *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011
  * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,13 +27,10 @@
 
 #include <stdint.h>
 
-#include "exc.h"
-
 
 namespace timer
 {
-    enum type
-    {
+    enum type {
         realtime,
         monotonic,
         process_cpu,
