@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Joe <cuchac@email.cz>
  *
@@ -91,6 +91,12 @@ public:
         set_fullscreen_flop_left,       // int
         set_fullscreen_flip_right,      // int
         set_fullscreen_flop_right,      // int
+        adjust_zoom,                    // float (relative adjustment)
+        set_zoom,                       // float (absolute value)
+        adjust_audio_volume,            // float (relative adjustment)
+        set_audio_volume,               // float (absolute value)
+        toggle_audio_mute,              // no parameters
+        set_audio_delay,                // float (absolute value)
     };
     
     type type;
@@ -122,6 +128,14 @@ public:
         param = oss.str();
     }
 
+    command(enum type t, int64_t p) :
+        type(t)
+    {
+        std::ostringstream oss;
+        s11n::save(oss, p);
+        param = oss.str();
+    }
+
     command(enum type t, const std::string &p) :
         type(t), param(p)
     {
@@ -165,6 +179,10 @@ public:
         fullscreen_flop_left,   // int
         fullscreen_flip_right,  // int
         fullscreen_flop_right,  // int
+        zoom,                   // float
+        audio_volume,           // float
+        audio_mute,             // int
+        audio_delay,            // float
     };
     
     type type;
@@ -209,6 +227,17 @@ public:
         current = ossc.str();
     }
 
+    notification(enum type t, int64_t p, int64_t c) :
+        type(t)
+    {
+        std::ostringstream ossp;
+        s11n::save(ossp, p);
+        previous = ossp.str();
+        std::ostringstream ossc;
+        s11n::save(ossc, c);
+        current = ossc.str();
+    }
+
     notification(enum type t, uint64_t p, uint64_t c) :
         type(t)
     {
@@ -249,6 +278,7 @@ public:
     void send_cmd(enum command::type t) { send_cmd(command(t)); }
     void send_cmd(enum command::type t, int p) { send_cmd(command(t, p)); }
     void send_cmd(enum command::type t, float p) { send_cmd(command(t, p)); }
+    void send_cmd(enum command::type t, int64_t p) { send_cmd(command(t, p)); }
     void send_cmd(enum command::type t, const std::string &p) { send_cmd(command(t, p)); }
 
     /* The controller receives notifications via this function. The default
@@ -270,6 +300,7 @@ public:
     static void notify_all(enum notification::type t, bool p, bool c) { notify_all(notification(t, p, c)); }
     static void notify_all(enum notification::type t, int p, int c) { notify_all(notification(t, p, c)); }
     static void notify_all(enum notification::type t, float p, float c) { notify_all(notification(t, p, c)); }
+    static void notify_all(enum notification::type t, int64_t p, int64_t c) { notify_all(notification(t, p, c)); }
     static void notify_all(enum notification::type t, uint64_t p, uint64_t c) { notify_all(notification(t, p, c)); }
     static void notify_all(enum notification::type t, const std::string &p, const std::string &c) { notify_all(notification(t, p, c)); }
 };
