@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Stefan Eilemann <eile@eyescale.ch>
  * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
@@ -25,6 +25,8 @@
 
 #include <cstring>
 #include <cstdlib>
+#include <cstdio>
+#include <cerrno>
 #include <limits>
 #include <locale.h>
 
@@ -145,6 +147,17 @@ static void qt_msg_handler(QtMsgType type, const char *msg)
     }
 }
 
+// Handle a log file that may be set via the --log-file option; see below
+static FILE *logf = NULL;
+static void close_log_file(void)
+{
+    if (logf)
+    {
+        (void)std::fclose(logf);
+    }
+}
+
+
 int main(int argc, char *argv[])
 {
     /* Initialization: gettext */
@@ -182,6 +195,8 @@ int main(int argc, char *argv[])
     options.push_back(&version);
     opt::flag no_gui("no-gui", 'n', opt::optional);
     options.push_back(&no_gui);
+    opt::val<std::string> log_file("log-file", '\0', opt::optional);
+    options.push_back(&log_file);
     std::vector<std::string> log_levels;
     log_levels.push_back("debug");
     log_levels.push_back("info");
@@ -190,6 +205,16 @@ int main(int argc, char *argv[])
     log_levels.push_back("quiet");
     opt::val<std::string> log_level("log-level", 'L', opt::optional, log_levels, "");
     options.push_back(&log_level);
+    opt::info list_audio_devices("list-audio-devices", '\0', opt::optional);
+    options.push_back(&list_audio_devices);
+    opt::val<int> audio_device("audio-device", 'A', opt::optional, 0, 999, 0);
+    options.push_back(&audio_device);
+    opt::val<int> audio_delay("audio-delay", 'D', opt::optional, -10000, +10000, 0);
+    options.push_back(&audio_delay);
+    opt::val<float> audio_volume("audio-volume", 'V', opt::optional, 0.0f, 1.0f, 1.0f);
+    options.push_back(&audio_volume);
+    opt::flag audio_mute("audio-mute", 'm', opt::optional);
+    options.push_back(&audio_mute);
     std::vector<std::string> device_types;
     device_types.push_back("default");
     device_types.push_back("firewire");
@@ -208,6 +233,8 @@ int main(int argc, char *argv[])
     input_modes.push_back("mono");
     input_modes.push_back("separate-left-right");
     input_modes.push_back("separate-right-left");
+    input_modes.push_back("alternating-left-right");
+    input_modes.push_back("alternating-right-left");
     input_modes.push_back("top-bottom");
     input_modes.push_back("top-bottom-half");
     input_modes.push_back("bottom-top");
@@ -270,6 +297,8 @@ int main(int argc, char *argv[])
     options.push_back(&fullscreen_flip_right);
     opt::flag fullscreen_flop_right("fullscreen-flop-right", '\0', opt::optional);
     options.push_back(&fullscreen_flop_right);
+    opt::val<float> zoom("zoom", 'z', opt::optional, 0.0f, 1.0f, parameters().zoom);
+    options.push_back(&zoom);
     opt::flag center("center", 'c', opt::optional);
     options.push_back(&center);
     opt::val<std::string> subtitle_encoding("subtitle-encoding", '\0', opt::optional, parameters().subtitle_encoding);
@@ -293,6 +322,8 @@ int main(int argc, char *argv[])
     options.push_back(&ghostbust);
     opt::flag benchmark("benchmark", 'b', opt::optional);
     options.push_back(&benchmark);
+    opt::val<int> swap_interval("swap-interval", '\0', opt::optional, 0, 999, -1);
+    options.push_back(&swap_interval);
     opt::flag loop("loop", 'l', opt::optional);
     options.push_back(&loop);
     // Accept some Equalizer options. These are passed to Equalizer for interpretation.
@@ -320,10 +351,25 @@ int main(int argc, char *argv[])
     {
         return 1;
     }
+    if (!log_file.value().empty())
+    {
+        logf = std::fopen(log_file.value().c_str(), "a");
+        if (!logf)
+        {
+            msg::err(_("%s: %s"), log_file.value().c_str(), std::strerror(errno));
+            return 1;
+        }
+        std::atexit(close_log_file);
+        msg::set_file(logf);
+        msg::set_columns(80);
+    }
+
     if (version.value())
     {
+        if (msg::file() == stderr)
+            msg::set_file(stdout);
         msg::req(_("%s version %s"), PACKAGE_NAME, VERSION);
-        msg::req(4, _("Copyright (C) 2011 the Bino developers."));
+        msg::req(4, _("Copyright (C) 2012 the Bino developers."));
         msg::req_txt(4, _("This is free software. You may redistribute copies of it "
                     "under the terms of the GNU General Public License. "
                     "There is NO WARRANTY, to the extent permitted by law."));
@@ -338,6 +384,8 @@ int main(int argc, char *argv[])
     }
     if (help.value())
     {
+        if (msg::file() == stderr)
+            msg::set_file(stdout);
         /* TRANSLATORS: This is the --help text. Translate only the description,
            not the option names. Please keep a proper indentation with spaces, and
            keep the line length limited. */
@@ -348,7 +396,13 @@ int main(int argc, char *argv[])
                     "  --help                   Print help.\n"
                     "  --version                Print version.\n"
                     "  -n|--no-gui              Do not use the GUI, just show a plain window.\n"
+                    "  --log-file=FILE          Append all log messages to the given file.\n"
                     "  -L|--log-level=LEVEL     Set log level (debug/info/warning/error/quiet).\n"
+                    "  --list-audio-devices     Print a list of known audio devices and exit.\n"
+                    "  -A|--audio-device=N      Use audio device number N (N=0 is the default).\n"
+                    "  -D|--audio-delay=D       Delay audio by D milliseconds. Default is 0.\n"
+                    "  -V|--audio-volume=V      Set audio volume (0 to 1). Default is 1.\n"
+                    "  -m|--audio-mute          Mute audio.\n"
                     "  --device-type=TYPE       Type of input device: default, firewire, x11.\n"
                     "  --device-frame-size=WxH  Request frame size WxH from input device.\n"
                     "  --device-frame-rate=N/D  Request frame rate N/D from input device.\n"
@@ -361,6 +415,8 @@ int main(int argc, char *argv[])
                     "    mono                     Single view.\n"
                     "    separate-left-right      Left/right separate streams, left first.\n"
                     "    separate-right-left      Left/right separate streams, right first.\n"
+                    "    alternating-left-right   Left/right alternating, left first.\n"
+                    "    alternating-right-left   Left/right alternating, right first.\n"
                     "    top-bottom               Left top, right bottom.\n"
                     "    top-bottom-half          Left top, right bottom, half height.\n"
                     "    bottom-top               Left bottom, right top.\n"
@@ -408,6 +464,7 @@ int main(int argc, char *argv[])
                     "  --fullscreen-flop-left   Flop left view horizontally when fullscreen.\n"
                     "  --fullscreen-flip-right  Flip right view vertically when fullscreen.\n"
                     "  --fullscreen-flop-right  Flop right view horizontally when fullscreen.\n"
+                    "  -z|--zoom=Z              Set zoom for wide videos (0=off to 1=full).\n"
                     "  -c|--center              Center window on screen.\n"
                     "  --subtitle-encoding=ENC  Set subtitle encoding.\n"
                     "  --subtitle-font=FONT     Set subtitle font name.\n"
@@ -420,6 +477,8 @@ int main(int argc, char *argv[])
                     "                           values for the R,G,B channels.\n"
                     "  -G|--ghostbust=VAL       Amount of ghostbusting to apply (0 to 1).\n"
                     "  -b|--benchmark           Benchmark mode (no audio, show fps).\n"
+                    "  --swap-interval=D        Frame rate divisor relative to display refresh\n"
+                    "                           rate, default 0 for benchmark, 1 otherwise.\n"
                     "  -l|--loop                Loop the input media.\n"
                     "\n"
                     "Interactive control:\n"
@@ -436,15 +495,35 @@ int main(int argc, char *argv[])
                     "  3, 4                     Adjust brightness.\n"
                     "  5, 6                     Adjust hue.\n"
                     "  7, 8                     Adjust saturation.\n"
-                    "  <, >                     Adjust parallax.\n"
+                    "  [, ]                     Adjust parallax.\n"
                     "  (, )                     Adjust ghostbusting.\n"
+                    "  <, >                     Adjust zoom for wide videos.\n"
+                    "  /, *                     Adjust audio volume.\n"
+                    "  m                        Toggle audio mute.\n"
                     "  left, right              Seek 10 seconds backward / forward.\n"
                     "  up, down                 Seek 1 minute backward / forward.\n"
                     "  page up, page down       Seek 10 minutes backward / forward.\n"
-                    "  Mouse click              Seek according to horizontal click position."),
+                    "  Mouse click              Seek according to horizontal click position.\n"
+                    "  Media keys               Media keys should work as expected."),
                 program_name);
     }
-    if (version.value() || help.value())
+    if (list_audio_devices.value())
+    {
+        audio_output ao;
+        if (ao.devices() == 0)
+        {
+            msg::req(_("No audio devices known."));
+        }
+        else
+        {
+            msg::req("%d audio devices available:", ao.devices());
+            for (int i = 0; i < ao.devices(); i++)
+            {
+                msg::req(4, "%d: %s", i + 1, ao.device_name(i).c_str());
+            }
+        }
+    }
+    if (version.value() || help.value() || list_audio_devices.value())
     {
         return 0;
     }
@@ -494,6 +573,10 @@ int main(int argc, char *argv[])
     {
         init_data.log_level = msg::REQ;
     }
+    if (audio_device.values().size() > 0)
+    {
+        init_data.audio_device = audio_device.value() - 1;
+    }
     if (device_type.value() == "")
     {
         init_data.dev_request.device =
@@ -576,6 +659,19 @@ int main(int argc, char *argv[])
     {
         init_data.params.fullscreen_flop_right = fullscreen_flop_right.value();
     }
+    init_data.params.zoom = zoom.value();
+    if (audio_delay.values().size() > 0)
+    {
+        init_data.params.audio_delay = audio_delay.value() * 1000;
+    }
+    if (audio_volume.values().size() > 0)
+    {
+        init_data.params.audio_volume = audio_volume.value();
+    }
+    if (audio_mute.values().size() > 0)
+    {
+        init_data.params.audio_mute = audio_mute.value();
+    }
     init_data.center = center.value();
     init_data.params.subtitle_encoding = subtitle_encoding.value();
     init_data.params.subtitle_font = subtitle_font.value();
@@ -594,8 +690,14 @@ int main(int argc, char *argv[])
     init_data.benchmark = benchmark.value();
     if (init_data.benchmark)
     {
+        init_data.swap_interval = 0;
         msg::inf(_("Benchmark mode: audio and time synchronization disabled."));
     }
+    if (swap_interval.value() >= 0)
+    {
+        init_data.swap_interval = swap_interval.value();
+    }
+
     init_data.params.loop_mode = (loop.value() ? parameters::loop_current : parameters::no_loop);
 
 #if HAVE_LIBLIRCCLIENT
@@ -622,6 +724,10 @@ int main(int argc, char *argv[])
         if (equalizer)
         {
 #if HAVE_LIBEQUALIZER
+            if (arguments.size() == 0)
+            {
+                throw exc(_("No video to play."));
+            }
             player = new class player_equalizer(&argc, argv, equalizer_flat_screen);
 #else
             throw exc(_("This version of Bino was compiled without support for Equalizer."));
