@@ -1,9 +1,10 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Joe <joe@wpj.cz>
+ * D. Matz <bandregent@yahoo.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -133,6 +134,9 @@ std::string video_frame::stereo_layout_to_string(stereo_layout_t stereo_layout,
     case separate:
         s = stereo_layout_swap ? "separate-right-left" : "separate-left-right";
         break;
+    case alternating:
+        s = stereo_layout_swap ? "alternating-right-left" : "alternating-left-right";
+        break;
     case top_bottom:
         s = stereo_layout_swap ? "bottom-top" : "top-bottom";
         break;
@@ -169,6 +173,16 @@ void video_frame::stereo_layout_from_string(const std::string &s, stereo_layout_
         stereo_layout = separate;
         stereo_layout_swap = false;
     }
+    else if (s == "alternating-right-left")
+    {
+        stereo_layout = alternating;
+        stereo_layout_swap = true;
+    }
+    else if (s == "alternating-left-right")
+    {
+        stereo_layout = alternating;
+        stereo_layout_swap = false;
+    }
     else if (s == "bottom-top")
     {
         stereo_layout = top_bottom;
@@ -267,6 +281,12 @@ std::string video_frame::format_name() const
         case u8_mpeg:
             name += "-mpeg";
             break;
+        case u10_full:
+            name += "-jpeg10";
+            break;
+        case u10_mpeg:
+            name += "-mpeg10";
+            break;
         }
     }
     if (layout == yuv422p || layout == yuv420p)
@@ -307,18 +327,19 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
     size_t dst_row_width = 0;
     size_t dst_row_size = 0;
     size_t lines = 0;
+    size_t type_size = (value_range == u8_full || value_range == u8_mpeg) ? 1 : 2;
 
     switch (layout)
     {
     case bgra32:
         dst_row_width = width * 4;
-        dst_row_size = dst_row_width;
+        dst_row_size = dst_row_width * type_size;
         lines = height;
         break;
 
     case yuv444p:
         dst_row_width = width;
-        dst_row_size = next_multiple_of_4(dst_row_width);
+        dst_row_size = next_multiple_of_4(dst_row_width * type_size);
         lines = height;
         break;
 
@@ -326,13 +347,13 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
         if (plane == 0)
         {
             dst_row_width = width;
-            dst_row_size = next_multiple_of_4(dst_row_width);
+            dst_row_size = next_multiple_of_4(dst_row_width * type_size);
             lines = height;
         }
         else
         {
             dst_row_width = width / 2;
-            dst_row_size = next_multiple_of_4(dst_row_width);
+            dst_row_size = next_multiple_of_4(dst_row_width * type_size);
             lines = height;
         }
         break;
@@ -341,13 +362,13 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
         if (plane == 0)
         {
             dst_row_width = width;
-            dst_row_size = next_multiple_of_4(dst_row_width);
+            dst_row_size = next_multiple_of_4(dst_row_width * type_size);
             lines = height;
         }
         else
         {
             dst_row_width = width / 2;
-            dst_row_size = next_multiple_of_4(dst_row_width);
+            dst_row_size = next_multiple_of_4(dst_row_width * type_size);
             lines = height / 2;
         }
         break;
@@ -365,6 +386,7 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
         src_offset = 0;
         break;
     case separate:
+    case alternating:
         src = static_cast<const char *>(data[view][plane]);
         src_row_size = line_size[view][plane];
         src_offset = 0;
@@ -388,6 +410,7 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
         break;
     }
 
+    assert(src);
     if (src_row_size == dst_row_size)
     {
         std::memcpy(dst, src + src_offset, lines * src_row_size);
@@ -397,7 +420,7 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
         size_t dst_offset = 0;
         for (size_t y = 0; y < lines; y++)
         {
-            std::memcpy(dst + dst_offset, src + src_offset, dst_row_width);
+            std::memcpy(dst + dst_offset, src + src_offset, dst_row_width * type_size);
             dst_offset += dst_row_size;
             src_offset += src_row_size;
         }
@@ -576,45 +599,49 @@ parameters::parameters() :
     fullscreen_flip_left(-1),
     fullscreen_flop_left(-1),
     fullscreen_flip_right(-1),
-    fullscreen_flop_right(-1)
+    fullscreen_flop_right(-1),
+    zoom(-1.0f),
+    audio_volume(std::numeric_limits<float>::quiet_NaN()),
+    audio_mute(-1),
+    audio_delay(std::numeric_limits<int64_t>::min())
 {
 }
 
 void parameters::set_defaults()
 {
-    if (!std::isfinite(parallax) || parallax < -1.0f || parallax > +1.0f)
+    if (!std::isnormal(parallax) || parallax < -1.0f || parallax > +1.0f)
     {
         parallax = 0.0f;
     }
-    if (!std::isfinite(crosstalk_r) || crosstalk_r < 0.0f || crosstalk_r > +1.0f)
+    if (!std::isnormal(crosstalk_r) || crosstalk_r < 0.0f || crosstalk_r > +1.0f)
     {
         crosstalk_r = 0.0f;
     }
-    if (!std::isfinite(crosstalk_g) || crosstalk_g < 0.0f || crosstalk_g > +1.0f)
+    if (!std::isnormal(crosstalk_g) || crosstalk_g < 0.0f || crosstalk_g > +1.0f)
     {
         crosstalk_g = 0.0f;
     }
-    if (!std::isfinite(crosstalk_b) || crosstalk_b < 0.0f || crosstalk_b > +1.0f)
+    if (!std::isnormal(crosstalk_b) || crosstalk_b < 0.0f || crosstalk_b > +1.0f)
     {
         crosstalk_b = 0.0f;
     }
-    if (!std::isfinite(ghostbust) || ghostbust < 0.0f || ghostbust > +1.0f)
+    if (!std::isnormal(ghostbust) || ghostbust < 0.0f || ghostbust > +1.0f)
     {
         ghostbust = 0.0f;
     }
-    if (!std::isfinite(contrast) || contrast < -1.0f || contrast > +1.0f)
+    if (!std::isnormal(contrast) || contrast < -1.0f || contrast > +1.0f)
     {
         contrast = 0.0f;
     }
-    if (!std::isfinite(brightness) || brightness < -1.0f || brightness > +1.0f)
+    if (!std::isnormal(brightness) || brightness < -1.0f || brightness > +1.0f)
     {
         brightness = 0.0f;
     }
-    if (!std::isfinite(hue) || hue < -1.0f || hue > +1.0f)
+    if (!std::isnormal(hue) || hue < -1.0f || hue > +1.0f)
     {
         hue = 0.0f;
     }
-    if (!std::isfinite(saturation) || saturation < -1.0f || saturation > +1.0f)
+    if (!std::isnormal(saturation) || saturation < -1.0f || saturation > +1.0f)
     {
         saturation = 0.0f;
     }
@@ -630,7 +657,7 @@ void parameters::set_defaults()
     {
         subtitle_size = -1;
     }
-    if (!std::isfinite(subtitle_scale) || subtitle_scale < 0.0f)
+    if (!std::isnormal(subtitle_scale) || subtitle_scale < 0.0f)
     {
         subtitle_scale = -1.0f;
     }
@@ -638,7 +665,7 @@ void parameters::set_defaults()
     {
         subtitle_color = std::numeric_limits<uint64_t>::max();
     }
-    if (!std::isfinite(subtitle_parallax) || subtitle_parallax < -1.0f || subtitle_parallax > +1.0f)
+    if (!std::isnormal(subtitle_parallax) || subtitle_parallax < -1.0f || subtitle_parallax > +1.0f)
     {
         subtitle_parallax = 0.0f;
     }
@@ -662,6 +689,22 @@ void parameters::set_defaults()
     {
         fullscreen_flop_right = 0;
     }
+    if (!std::isnormal(zoom) || zoom < 0.0f)
+    {
+        zoom = 0.0f;
+    }
+    if (!std::isnormal(audio_volume) || audio_volume < 0.0f || audio_volume > 1.0f)
+    {
+        audio_volume = 1.0f;
+    }
+    if (audio_mute != 0 && audio_mute != 1)
+    {
+        audio_mute = 0;
+    }
+    if (audio_delay == std::numeric_limits<int64_t>::min())
+    {
+        audio_delay = 0;
+    }
 }
 
 std::string parameters::stereo_mode_to_string(stereo_mode_t stereo_mode, bool stereo_mode_swap)
@@ -898,6 +941,10 @@ void parameters::save(std::ostream &os) const
     s11n::save(os, fullscreen_flop_left);
     s11n::save(os, fullscreen_flip_right);
     s11n::save(os, fullscreen_flop_right);
+    s11n::save(os, zoom);
+    s11n::save(os, audio_volume);
+    s11n::save(os, audio_mute);
+    s11n::save(os, audio_delay);
 }
 
 void parameters::load(std::istream &is)
@@ -928,4 +975,8 @@ void parameters::load(std::istream &is)
     s11n::load(is, fullscreen_flop_left);
     s11n::load(is, fullscreen_flip_right);
     s11n::load(is, fullscreen_flop_right);
+    s11n::load(is, zoom);
+    s11n::load(is, audio_volume);
+    s11n::load(is, audio_mute);
+    s11n::load(is, audio_delay);
 }
