@@ -1,9 +1,10 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Joe <joe@wpj.cz>
+ * D. Matz <bandregent@yahoo.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +29,7 @@
 #include "s11n.h"
 
 
-class device_request : public s11n
+class device_request : public serializable
 {
 public:
     typedef enum
@@ -83,7 +84,9 @@ public:
     typedef enum
     {
         u8_full,        // 0-255 for all components
-        u8_mpeg         // 16-235 for Y, 16-240 for U and V
+        u8_mpeg,        // 16-235 for Y, 16-240 for U and V
+        u10_full,       // 0-1023 for all components (stored in 16 bits)
+        u10_mpeg,       // 64-940 for Y, 64-960 for U and V (stored in 16 bits)
     } value_range_t;
 
     // Location of chroma samples (only relevant for chroma subsampling layouts)
@@ -99,6 +102,7 @@ public:
     {
         mono,           // 1 video source: center view
         separate,       // 2 video sources: left and right view independent
+        alternating,    // 2 video sources: left and right view consecutively
         top_bottom,     // 1 video source: left view top, right view bottom, both with full size
         top_bottom_half,// 1 video source: left view top, right view bottom, both with half size
         left_right,     // 1 video source: left view left, right view right, both with full size
@@ -192,7 +196,7 @@ public:
     int sample_bits() const;
 };
 
-class subtitle_box : public s11n
+class subtitle_box : public serializable
 {
 public:
     // Format
@@ -204,7 +208,7 @@ public:
     } format_t;
 
     // Image data
-    class image_t : public s11n
+    class image_t : public serializable
     {
     public:
         int w, h;                       // Dimensions
@@ -268,7 +272,7 @@ public:
     void load(std::istream &is);
 };
 
-class parameters : public s11n
+class parameters : public serializable
 {
 public:
     // Do not change these enum values without a good reason, because the GUI saves them to a file.
@@ -330,6 +334,10 @@ public:
     int fullscreen_flop_left;           // Flop left view horizontally in fullscreen mode (0/1)
     int fullscreen_flip_right;          // Flip right view vertically in fullscreen mode (0/1)
     int fullscreen_flop_right;          // Flop right view horizontally in fullscreen mode (0/1)
+    float zoom;                         // Zoom, 0 = off (show full video width) .. 1 = full (use full screen height)
+    float audio_volume;                 // Audio volume, 0 .. 1
+    int audio_mute;                     // Audio mute: -1 = unknown, 0 = off, 1 = on
+    int64_t audio_delay;                // Audio delay in microseconds. INT64_MIN means "unspecified".
 
     // Constructor
     parameters();
