@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Alexey Osipov <lion-simba@pridelands.ru>
  * Joe <cuchac@email.cz>
@@ -43,11 +43,13 @@
 
 player_init_data::player_init_data() :
     log_level(msg::INF),
+    audio_device(-2),
     dev_request(),
     urls(),
     video_stream(0),
     audio_stream(0),
     subtitle_stream(-1),
+    swap_interval(1),
     benchmark(false),
     fullscreen(false),
     center(false),
@@ -68,11 +70,13 @@ player_init_data::~player_init_data()
 void player_init_data::save(std::ostream &os) const
 {
     s11n::save(os, static_cast<int>(log_level));
+    s11n::save(os, audio_device);
     s11n::save(os, dev_request);
     s11n::save(os, urls);
     s11n::save(os, video_stream);
     s11n::save(os, audio_stream);
     s11n::save(os, subtitle_stream);
+    s11n::save(os, swap_interval);
     s11n::save(os, benchmark);
     s11n::save(os, fullscreen);
     s11n::save(os, center);
@@ -90,11 +94,13 @@ void player_init_data::load(std::istream &is)
     int x;
     s11n::load(is, x);
     log_level = static_cast<msg::level_t>(x);
+    s11n::load(is, audio_device);
     s11n::load(is, dev_request);
     s11n::load(is, urls);
     s11n::load(is, video_stream);
     s11n::load(is, audio_stream);
     s11n::load(is, subtitle_stream);
+    s11n::load(is, swap_interval);
     s11n::load(is, benchmark);
     s11n::load(is, fullscreen);
     s11n::load(is, center);
@@ -174,7 +180,7 @@ void player::stop_playback()
 
 video_output *player::create_video_output()
 {
-    return new video_output_qt(_benchmark);
+    return new video_output_qt(_swap_interval);
 }
 
 void player::destroy_video_output(video_output *vo)
@@ -202,6 +208,7 @@ void player::open(const player_init_data &init_data)
     // Initialize basics
     msg::set_level(init_data.log_level);
     _benchmark = init_data.benchmark;
+    _swap_interval = init_data.swap_interval;
     reset_playstate();
 
     // Create media input
@@ -248,7 +255,7 @@ void player::open(const player_init_data &init_data)
     }
     if (_audio_output)
     {
-        _audio_output->init();
+        _audio_output->init(init_data.audio_device < -1 ? -1 : init_data.audio_device);
     }
 
     // Create video output
@@ -284,6 +291,10 @@ void player::open(const player_init_data &init_data)
     }
 
     // Set initial parameters
+    if (_audio_output)
+    {
+        _audio_output->set_parameters(_params);
+    }
     if (_video_output)
     {
         _video_output->set_parameters(_params);
@@ -634,11 +645,13 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         }
 
         int64_t allowable_sleep = 0;
-        if (_master_time_current >= _video_pos || _benchmark || _media_input->is_device())
+        if (_master_time_current + _params.audio_delay
+                >= _video_pos || _benchmark || _media_input->is_device())
         {
             // Output current video frame
             _drop_next_frame = false;
-            if (_master_time_current - _video_pos > _media_input->video_frame_duration() * 75 / 100
+            if (_master_time_current + _params.audio_delay - _video_pos
+                    > _media_input->video_frame_duration() * 75 / 100
                     && !_benchmark && !_media_input->is_device())
             {
                 msg::wrn(_("Video: delay %g seconds; dropping next frame."), (_master_time_current - _video_pos) / 1e6f);
@@ -1130,10 +1143,58 @@ void player::receive_cmd(const command &cmd)
             controller::notify_all(notification::fullscreen_flop_right, old, _params.fullscreen_flop_right);
         }
         break;
+    case command::adjust_zoom:
+        s11n::load(p, param);
+        oldval = _params.zoom;
+        _params.zoom = std::max(std::min(_params.zoom + param, 1.0f), 0.0f);
+        parameters_changed = true;
+        controller::notify_all(notification::zoom, oldval, _params.zoom);
+        break;
+    case command::set_zoom:
+        s11n::load(p, param);
+        oldval = _params.zoom;
+        _params.zoom = std::max(std::min(param, 1.0f), 0.0f);
+        parameters_changed = true;
+        controller::notify_all(notification::zoom, oldval, _params.zoom);
+        break;
+    case command::adjust_audio_volume:
+        s11n::load(p, param);
+        oldval = _params.audio_volume;
+        _params.audio_volume = std::max(std::min(_params.audio_volume + param, 1.0f), 0.0f);
+        parameters_changed = true;
+        controller::notify_all(notification::audio_volume, oldval, _params.audio_volume);
+        break;
+    case command::set_audio_volume:
+        s11n::load(p, param);
+        oldval = _params.audio_volume;
+        _params.audio_volume = std::max(std::min(param, 1.0f), 0.0f);
+        parameters_changed = true;
+        controller::notify_all(notification::audio_volume, oldval, _params.audio_volume);
+        break;
+    case command::toggle_audio_mute:
+        _params.audio_mute = !_params.audio_mute;
+        parameters_changed = true;
+        controller::notify_all(notification::audio_mute, _params.audio_mute ? 0 : 1, _params.audio_mute ? 1 : 0);
+        break;
+    case command::set_audio_delay:
+        {
+            int64_t old = _params.audio_delay;
+            s11n::load(p, _params.audio_delay);
+            parameters_changed = true;
+            controller::notify_all(notification::audio_delay, old, _params.audio_delay);
+        }
+        break;
     }
 
-    if (parameters_changed && _video_output)
+    if (parameters_changed)
     {
-        _video_output->set_parameters(_params);
+        if (_audio_output)
+        {
+            _audio_output->set_parameters(_params);
+        }
+        if (_video_output)
+        {
+            _video_output->set_parameters(_params);
+        }
     }
 }
