@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Alexey Osipov <lion-simba@pridelands.ru>
  * Joe <cuchac@email.cz>
@@ -37,15 +37,17 @@
 
 /* The player_init_data contains everything that a player needs to start. */
 
-class player_init_data : public s11n
+class player_init_data : public serializable
 {
 public:
     msg::level_t log_level;                     // Level of log messages
+    int audio_device;                           // Audio device index, or -1 for default
     device_request dev_request;                 // Request for input device settings
     std::vector<std::string> urls;              // Input media objects
     int video_stream;                           // Selected video stream
     int audio_stream;                           // Selected audio stream
     int subtitle_stream;                        // Selected subtitle stream
+    int swap_interval;                          // locked refresh rate
     bool benchmark;                             // Benchmark mode?
     bool fullscreen;                            // Make video fullscreen?
     bool center;                                // Center video on screen?
@@ -98,6 +100,7 @@ private:
 
     // Benchmark mode
     bool _benchmark;                            // Is benchmark mode active?
+    int _swap_interval;                         // 0 in benchmark mode
     int _frames_shown;                          // Frames shown since last reset
     int64_t _fps_mark_time;                     // Time when _frames_shown was reset to zero
 
