@@ -1,10 +1,11 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
  * Joe <cuchac@email.cz>
+ * Daniel Schaal <farbing@web.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -115,6 +116,7 @@ class controls_widget : public QWidget, public controller
     Q_OBJECT
 
 private:
+    parameters *_params;
     int64_t _input_duration;
     bool _lock;
     QSettings *_settings;
@@ -132,8 +134,13 @@ private:
     QPushButton *_fff_button;
     QSlider *_seek_slider;
     QLabel *_pos_label;
+    QPushButton *_audio_mute_button;
+    QSlider *_audio_volume_slider;
     bool _playing;
 
+private:
+    void update_audio_widgets();
+
 private slots:
     void play_pressed();
     void pause_pressed();
@@ -148,9 +155,11 @@ private slots:
     void ff_pressed();
     void fff_pressed();
     void seek_slider_changed();
+    void audio_mute_pressed();
+    void audio_volume_slider_changed();
 
 public:
-    controls_widget(QSettings *settings, const player_init_data &init_data, QWidget *parent);
+    controls_widget(parameters *params, QSettings *settings, const player_init_data &init_data, QWidget *parent);
     virtual ~controls_widget();
 
     void update(const player_init_data &init_data, bool have_valid_input, bool playing, int64_t input_duration);
@@ -209,6 +218,27 @@ public:
     virtual void receive_notification(const notification &note);
 };
 
+class audio_dialog : public QDialog, public controller
+{
+    Q_OBJECT
+
+private:
+    player_init_data *_init_data;
+    parameters *_params;
+    bool _lock;
+    QComboBox *_device_combobox;
+    QSpinBox *_delay_spinbox;
+
+private slots:
+    void device_changed();
+    void delay_changed();
+
+public:
+    audio_dialog(player_init_data *init_data, parameters *params, QWidget *parent);
+
+    virtual void receive_notification(const notification &note);
+};
+
 class subtitle_dialog: public QDialog, public controller
 {
     Q_OBJECT
@@ -245,13 +275,15 @@ public:
     virtual void receive_notification(const notification &note);
 };
 
-class stereoscopic_dialog : public QDialog, public controller
+class video_dialog : public QDialog, public controller
 {
     Q_OBJECT
 
 private:
     parameters *_params;
     bool _lock;
+    QDoubleSpinBox *_z_spinbox;
+    QSlider *_z_slider;
     QDoubleSpinBox *_p_spinbox;
     QSlider *_p_slider;
     QDoubleSpinBox *_sp_spinbox;
@@ -260,6 +292,8 @@ private:
     QSlider *_g_slider;
 
 private slots:
+    void z_slider_changed(int val);
+    void z_spinbox_changed(double val);
     void p_slider_changed(int val);
     void p_spinbox_changed(double val);
     void sp_slider_changed(int val);
@@ -268,7 +302,7 @@ private slots:
     void g_spinbox_changed(double val);
 
 public:
-    stereoscopic_dialog(parameters *params, QWidget *parent);
+    video_dialog(parameters *params, QWidget *parent);
 
     virtual void receive_notification(const notification &note);
 };
@@ -312,17 +346,26 @@ private:
     controls_widget *_controls_widget;
     color_dialog *_color_dialog;
     crosstalk_dialog *_crosstalk_dialog;
+    audio_dialog *_audio_dialog;
     subtitle_dialog *_subtitle_dialog;
-    stereoscopic_dialog *_stereoscopic_dialog;
+    video_dialog *_video_dialog;
     player_qt_internal *_player;
     QTimer *_timer;
     player_init_data _init_data;
     const player_init_data _init_data_template;
     bool _stop_request;
 
+    int _max_recent_files;
+    QList<QAction *> _recent_file_actions;
+    QAction *_recent_files_separator;
+    QAction *_clear_recent_separator;
+    QAction *_clear_recent_files_act;
+
     QString current_file_hash();
     bool open_player();
     void open(QStringList urls, const device_request &dev_request = device_request());
+    void update_recent_file_actions();
+    QString stripped_name(const QStringList & filenames);
 
 private slots:
     void move_event();
@@ -330,16 +373,20 @@ private slots:
     void file_open();
     void file_open_url();
     void file_open_device();
+    void preferences_fullscreen();
     void preferences_colors();
     void preferences_crosstalk();
+    void preferences_audio();
     void preferences_subtitle();
-    void preferences_stereoscopic();
-    void preferences_fullscreen();
+    void preferences_video();
     void help_manual();
     void help_website();
     void help_keyboard();
     void help_about();
 
+    void open_recent_file();
+    void clear_recent_files();
+
 protected:
     void dragEnterEvent(QDragEnterEvent *event);
     void dropEvent(QDropEvent *event);
