@@ -24,6 +24,7 @@
 
 #include <limits>
 #include <cstdlib>
+#include <cstring>
 #include <cmath>
 
 #include <GL/glew.h>
@@ -58,23 +59,29 @@
  * The input data is first converted to YUV (for the common planar YUV frame
  * formats, this just means gathering of the three components from the
  * three planes). Then color adjustment in the YUV space is performed.
- * Finally the result is converted to sRGB and stored in an GL_SRGB texture.
+ * If the input data had an 8 bit value range, the result is converted to sRGB
+ * and stored in an GL_SRGB texture. If the input data had a larger value range,
+ * the result is converted to linear RGB and stored in an GL_RGB16 texture.
  * In this color correction step, no interpolation is done, because we're
  * dealing with non-linear values, and interpolating them would lead to
- * errors. We do not convert to linear RGB (as opposed to sRGB) in this step
- * because storing linear RGB in a GL_RGB texture would lose some precision
- * when compared to the non-linear input data.
+ * errors. We do not store linear RGB in GL_RGB8 textures because that would
+ * lose some precision when compared to the input data - so we either use
+ * GL_SRGB8 (and store sRGB values) or GL_RGB16 (and store linear values).
+ * In both cases, the rendering step can properly interpolate.
  *
  * Step 3: Rendering.
- * This step reads from the sRGB textures created in the previous step, which
- * means that the GL will transform the input to linear RGB automatically and
- * handle hardware accelerated bilinear interpolation correctly. Thus,
- * magnification or minification are safe in this step. Furthermore, we can
- * do interpolation on the linear RGB values for the masking output modes.
- * We then transform the resulting linear RGB values back to non-linear sRGB
- * values for output. We do not use the GL_ARB_framebuffer_sRGB extension for
- * this purpose because 1) we need computations on non-linear values for the
- * anaglyph methods and 2) sRGB framebuffers are not yet widely supported.
+ * This step reads from the color textures created in the previous step. In the
+ * case of GL_SRGB8 textures, this means that OpenGL will transform the input to
+ * linear RGB automatically and handle hardware accelerated bilinear
+ * interpolation correctly. Thus, magnification or minification are safe in this
+ * step. With GL_RGB16 textures and the linear values stored therein, no special
+ * handling is necessary.
+ * Furthermore, we can do interpolation on the linear RGB values for the masking
+ * output modes. We then transform the resulting linear RGB values back to
+ * non-linear sRGB values for output. We do not use the GL_ARB_framebuffer_sRGB
+ * extension for this purpose because 1) we need computations on non-linear
+ * values for the anaglyph methods and 2) sRGB framebuffers are not yet widely
+ * supported.
  *
  * Open issues / TODO:
  * The 420p and 422p chroma subsampling formats are currently handled by
@@ -85,11 +92,14 @@
  * interpolation, too.
  */
 
+static const float full_tex_coords[2][4][2] =
+{
+    { { 0.0f, 0.0f }, { 1.0f, 0.0f }, { 1.0f, 1.0f }, { 0.0f, 1.0f } },
+    { { 0.0f, 0.0f }, { 1.0f, 0.0f }, { 1.0f, 1.0f }, { 0.0f, 1.0f } }
+};
+
 video_output::video_output() : controller(), _initialized(false)
 {
-    // XXX: Hack: work around broken SRGB texture implementations
-    _srgb_textures_are_broken = std::getenv("SRGB_TEXTURES_ARE_BROKEN");
-
     _input_pbo = 0;
     _input_fbo = 0;
     _active_index = 1;
@@ -102,7 +112,7 @@ video_output::video_output() : controller(), _initialized(false)
             _input_yuv_v_tex[i][j] = 0;
             _input_bgra32_tex[i][j] = 0;
         }
-        _color_srgb_tex[i] = 0;
+        _color_tex[i] = 0;
         _input_subtitle_tex[i] = 0;
         _input_subtitle_width[i] = -1;
         _input_subtitle_height[i] = -1;
@@ -212,6 +222,9 @@ void video_output::input_init(int index, const video_frame &frame)
             _input_yuv_chroma_height_divisor[index] = 2;
             need_chroma_filtering = true;
         }
+        bool type_u8 = (frame.value_range == video_frame::u8_full || frame.value_range == video_frame::u8_mpeg);
+        GLint internal_format = type_u8 ? GL_LUMINANCE8 : GL_LUMINANCE16;
+        GLint type = type_u8 ? GL_UNSIGNED_BYTE : GL_UNSIGNED_SHORT;
         for (int i = 0; i < (frame.stereo_layout == video_frame::mono ? 1 : 2); i++)
         {
             glGenTextures(1, &(_input_yuv_y_tex[index][i]));
@@ -220,30 +233,30 @@ void video_output::input_init(int index, const video_frame &frame)
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-            glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8,
+            glTexImage2D(GL_TEXTURE_2D, 0, internal_format,
                     frame.width,
                     frame.height,
-                    0, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
+                    0, GL_LUMINANCE, type, NULL);
             glGenTextures(1, &(_input_yuv_u_tex[index][i]));
             glBindTexture(GL_TEXTURE_2D, _input_yuv_u_tex[index][i]);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, need_chroma_filtering ? GL_LINEAR : GL_NEAREST);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, need_chroma_filtering ? GL_LINEAR : GL_NEAREST);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-            glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8,
+            glTexImage2D(GL_TEXTURE_2D, 0, internal_format,
                     frame.width / _input_yuv_chroma_width_divisor[index],
                     frame.height / _input_yuv_chroma_height_divisor[index],
-                    0, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
+                    0, GL_LUMINANCE, type, NULL);
             glGenTextures(1, &(_input_yuv_v_tex[index][i]));
             glBindTexture(GL_TEXTURE_2D, _input_yuv_v_tex[index][i]);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, need_chroma_filtering ? GL_LINEAR : GL_NEAREST);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, need_chroma_filtering ? GL_LINEAR : GL_NEAREST);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-            glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8,
+            glTexImage2D(GL_TEXTURE_2D, 0, internal_format,
                     frame.width / _input_yuv_chroma_width_divisor[index],
                     frame.height / _input_yuv_chroma_height_divisor[index],
-                    0, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
+                    0, GL_LUMINANCE, type, NULL);
         }
     }
     assert(xgl::CheckError(HERE));
@@ -326,9 +339,16 @@ void video_output::prepare_next_frame(const video_frame &frame, const subtitle_b
         input_init(index, frame);
     }
     _frame[index] = frame;
-    int bytes_per_pixel = (frame.layout == video_frame::bgra32 ? 4 : 1);
-    GLenum format = (frame.layout == video_frame::bgra32 ? GL_BGRA : GL_LUMINANCE);
-    GLenum type = (frame.layout == video_frame::bgra32 ? GL_UNSIGNED_INT_8_8_8_8_REV : GL_UNSIGNED_BYTE);
+    int bytes_per_pixel = 4;
+    GLenum format = GL_BGRA;
+    GLenum type = GL_UNSIGNED_INT_8_8_8_8_REV;
+    if (frame.layout != video_frame::bgra32)
+    {
+        bool type_u8 = (frame.value_range == video_frame::u8_full || frame.value_range == video_frame::u8_mpeg);
+        bytes_per_pixel = type_u8 ? 1 : 2;
+        format = GL_LUMINANCE;
+        type = type_u8 ? GL_UNSIGNED_BYTE : GL_UNSIGNED_SHORT;
+    }
     for (int i = 0; i < (frame.stereo_layout == video_frame::mono ? 1 : 2); i++)
     {
         for (int plane = 0; plane < (frame.layout == video_frame::bgra32 ? 1 : 3); plane++)
@@ -501,6 +521,7 @@ void video_output::color_init(const video_frame &frame)
     std::string layout_str;
     std::string color_space_str;
     std::string value_range_str;
+    std::string storage_str;
     std::string chroma_offset_x_str;
     std::string chroma_offset_y_str;
     if (frame.layout == video_frame::bgra32)
@@ -508,6 +529,7 @@ void video_output::color_init(const video_frame &frame)
         layout_str = "layout_bgra32";
         color_space_str = "color_space_srgb";
         value_range_str = "value_range_8bit_full";
+        storage_str = "storage_srgb";
     }
     else
     {
@@ -523,10 +545,22 @@ void video_output::color_init(const video_frame &frame)
         if (frame.value_range == video_frame::u8_full)
         {
             value_range_str = "value_range_8bit_full";
+            storage_str = "storage_srgb";
+        }
+        else if (frame.value_range == video_frame::u8_mpeg)
+        {
+            value_range_str = "value_range_8bit_mpeg";
+            storage_str = "storage_srgb";
+        }
+        else if (frame.value_range == video_frame::u10_full)
+        {
+            value_range_str = "value_range_10bit_full";
+            storage_str = "storage_linear_rgb";
         }
         else
         {
-            value_range_str = "value_range_8bit_mpeg";
+            value_range_str = "value_range_10bit_mpeg";
+            storage_str = "storage_linear_rgb";
         }
         chroma_offset_x_str = "0.0";
         chroma_offset_y_str = "0.0";
@@ -561,24 +595,32 @@ void video_output::color_init(const video_frame &frame)
             }
         }
     }
+    // XXX: Hack: work around broken SRGB texture implementations
+    if (std::getenv("SRGB_TEXTURES_ARE_BROKEN"))
+    {
+        msg::dbg("Avoiding broken SRGB texture implementation.");
+        storage_str = "storage_linear_rgb";
+    }
+
     std::string color_fs_src(VIDEO_OUTPUT_COLOR_FS_GLSL_STR);
     str::replace(color_fs_src, "$layout", layout_str);
     str::replace(color_fs_src, "$color_space", color_space_str);
     str::replace(color_fs_src, "$value_range", value_range_str);
     str::replace(color_fs_src, "$chroma_offset_x", chroma_offset_x_str);
     str::replace(color_fs_src, "$chroma_offset_y", chroma_offset_y_str);
+    str::replace(color_fs_src, "$storage", storage_str);
     _color_prg = xgl::CreateProgram("video_output_color", "", "", color_fs_src);
     xgl::LinkProgram("video_output_color", _color_prg);
     for (int i = 0; i < (frame.stereo_layout == video_frame::mono ? 1 : 2); i++)
     {
-        glGenTextures(1, &(_color_srgb_tex[i]));
-        glBindTexture(GL_TEXTURE_2D, _color_srgb_tex[i]);
+        glGenTextures(1, &(_color_tex[i]));
+        glBindTexture(GL_TEXTURE_2D, _color_tex[i]);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
         glTexImage2D(GL_TEXTURE_2D, 0,
-                _srgb_textures_are_broken ? GL_RGB8 : GL_SRGB8,
+                storage_str == "storage_srgb" ? GL_SRGB8 : GL_RGB16,
                 frame.width, frame.height, 0, GL_BGRA, GL_UNSIGNED_INT_8_8_8_8_REV, NULL);
     }
     assert(xgl::CheckError(HERE));
@@ -596,10 +638,10 @@ void video_output::color_deinit()
     }
     for (int i = 0; i < 2; i++)
     {
-        if (_color_srgb_tex[i] != 0)
+        if (_color_tex[i] != 0)
         {
-            glDeleteTextures(1, &(_color_srgb_tex[i]));
-            _color_srgb_tex[i] = 0;
+            glDeleteTextures(1, &(_color_tex[i]));
+            _color_tex[i] = 0;
         }
     }
     _color_last_frame = video_frame();
@@ -639,10 +681,8 @@ void video_output::render_init()
             : _params.stereo_mode == parameters::red_green_monochrome ? "mode_red_green_monochrome"
             : _params.stereo_mode == parameters::red_blue_monochrome ? "mode_red_blue_monochrome"
             : "mode_onechannel");
-    std::string srgb_broken_str = (_srgb_textures_are_broken ? "1" : "0");
     std::string render_fs_src(VIDEO_OUTPUT_RENDER_FS_GLSL_STR);
     str::replace(render_fs_src, "$mode", mode_str);
-    str::replace(render_fs_src, "$srgb_broken", srgb_broken_str);
     _render_prg = xgl::CreateProgram("video_output_render", "", "", render_fs_src);
     xgl::LinkProgram("video_output_render", _render_prg);
     uint32_t dummy_texture = 0;
@@ -725,33 +765,35 @@ void video_output::set_parameters(const parameters &params)
 }
 
 static void draw_quad(float x, float y, float w, float h,
-        const float tex_coords[2][4][2],
+        const float tex_coords[2][4][2] = NULL,
         const float more_tex_coords[4][2] = NULL)
 {
+    const float (*my_tex_coords)[4][2] = (tex_coords ? tex_coords : full_tex_coords);
+
     glBegin(GL_QUADS);
-    glTexCoord2f(tex_coords[0][0][0], tex_coords[0][0][1]);
-    glMultiTexCoord2f(GL_TEXTURE1, tex_coords[1][0][0], tex_coords[1][0][1]);
+    glTexCoord2f(my_tex_coords[0][0][0], my_tex_coords[0][0][1]);
+    glMultiTexCoord2f(GL_TEXTURE1, my_tex_coords[1][0][0], my_tex_coords[1][0][1]);
     if (more_tex_coords)
     {
         glMultiTexCoord2f(GL_TEXTURE2, more_tex_coords[0][0], more_tex_coords[0][1]);
     }
     glVertex2f(x, y);
-    glTexCoord2f(tex_coords[0][1][0], tex_coords[0][1][1]);
-    glMultiTexCoord2f(GL_TEXTURE1, tex_coords[1][1][0], tex_coords[1][1][1]);
+    glTexCoord2f(my_tex_coords[0][1][0], my_tex_coords[0][1][1]);
+    glMultiTexCoord2f(GL_TEXTURE1, my_tex_coords[1][1][0], my_tex_coords[1][1][1]);
     if (more_tex_coords)
     {
         glMultiTexCoord2f(GL_TEXTURE2, more_tex_coords[1][0], more_tex_coords[1][1]);
     }
     glVertex2f(x + w, y);
-    glTexCoord2f(tex_coords[0][2][0], tex_coords[0][2][1]);
-    glMultiTexCoord2f(GL_TEXTURE1, tex_coords[1][2][0], tex_coords[1][2][1]);
+    glTexCoord2f(my_tex_coords[0][2][0], my_tex_coords[0][2][1]);
+    glMultiTexCoord2f(GL_TEXTURE1, my_tex_coords[1][2][0], my_tex_coords[1][2][1]);
     if (more_tex_coords)
     {
         glMultiTexCoord2f(GL_TEXTURE2, more_tex_coords[2][0], more_tex_coords[2][1]);
     }
     glVertex2f(x + w, y + h);
-    glTexCoord2f(tex_coords[0][3][0], tex_coords[0][3][1]);
-    glMultiTexCoord2f(GL_TEXTURE1, tex_coords[1][3][0], tex_coords[1][3][1]);
+    glTexCoord2f(my_tex_coords[0][3][0], my_tex_coords[0][3][1]);
+    glMultiTexCoord2f(GL_TEXTURE1, my_tex_coords[1][3][0], my_tex_coords[1][3][1]);
     if (more_tex_coords)
     {
         glMultiTexCoord2f(GL_TEXTURE2, more_tex_coords[3][0], more_tex_coords[3][1]);
@@ -762,7 +804,9 @@ static void draw_quad(float x, float y, float w, float h,
 
 void video_output::display_current_frame(
         bool keep_viewport, bool mono_right_instead_of_left,
-        float x, float y, float w, float h, const GLint viewport[2][4])
+        float x, float y, float w, float h,
+        const GLint viewport[2][4],
+        const float tex_coords[2][4][2])
 {
     make_context_current();
     assert(xgl::CheckError(HERE));
@@ -778,7 +822,9 @@ void video_output::display_current_frame(
                 || frame.height != _color_last_frame.height
                 || frame.aspect_ratio < _color_last_frame.aspect_ratio
                 || frame.aspect_ratio > _color_last_frame.aspect_ratio
-                || _render_last_params.stereo_mode != _params.stereo_mode))
+                || _render_last_params.stereo_mode != _params.stereo_mode
+                || _render_last_params.zoom < _params.zoom
+                || _render_last_params.zoom > _params.zoom))
     {
         reshape(width(), height());
     }
@@ -820,11 +866,6 @@ void video_output::display_current_frame(
 
     glEnable(GL_TEXTURE_2D);
     glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-    float tex_coords[2][4][2] =
-    {
-        { { 0.0f, 0.0f }, { 1.0f, 0.0f }, { 1.0f, 1.0f }, { 0.0f, 1.0f } },
-        { { 0.0f, 0.0f }, { 1.0f, 0.0f }, { 1.0f, 1.0f }, { 0.0f, 1.0f } }
-    };
 
     /* Step 2: color-correction */
 
@@ -854,7 +895,7 @@ void video_output::display_current_frame(
     glUniform1f(glGetUniformLocation(_color_prg, "cos_hue"), std::cos(_params.hue * M_PI));
     glUniform1f(glGetUniformLocation(_color_prg, "sin_hue"), std::sin(_params.hue * M_PI));
     glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, _color_fbo);
-    // left view: render into _color_srgb_tex[0]
+    // left view: render into _color_tex[0]
     if (frame.layout == video_frame::bgra32)
     {
         glActiveTexture(GL_TEXTURE0);
@@ -870,9 +911,9 @@ void video_output::display_current_frame(
         glBindTexture(GL_TEXTURE_2D, _input_yuv_v_tex[_active_index][left]);
     }
     glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
-            GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _color_srgb_tex[0], 0);
-    draw_quad(-1.0f, +1.0f, +2.0f, -2.0f, tex_coords);
-    // right view: render into _color_srgb_tex[1]
+            GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _color_tex[0], 0);
+    draw_quad(-1.0f, +1.0f, +2.0f, -2.0f);
+    // right view: render into _color_tex[1]
     if (left != right)
     {
         if (frame.layout == video_frame::bgra32)
@@ -890,8 +931,8 @@ void video_output::display_current_frame(
             glBindTexture(GL_TEXTURE_2D, _input_yuv_v_tex[_active_index][right]);
         }
         glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,
-                GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _color_srgb_tex[1], 0);
-        draw_quad(-1.0f, +1.0f, +2.0f, -2.0f, tex_coords);
+                GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, _color_tex[1], 0);
+        draw_quad(-1.0f, +1.0f, +2.0f, -2.0f);
     }
     glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
     glViewport(viewport[0][0], viewport[0][1], viewport[0][2], viewport[0][3]);
@@ -904,43 +945,45 @@ void video_output::display_current_frame(
         glEnable(GL_SCISSOR_TEST);
     }
 
-    // at this point, the left view is in _color_srgb_tex[0],
-    // and the right view (if it exists) is in _color_srgb_tex[1]
+    // at this point, the left view is in _color_tex[0],
+    // and the right view (if it exists) is in _color_tex[1]
     right = (left != right ? 1 : 0);
     left = 0;
 
     /* Step 3: rendering */
 
     // Apply fullscreen flipping/flopping
+    float my_tex_coords[2][4][2];
+    std::memcpy(my_tex_coords, tex_coords, sizeof(my_tex_coords));
     if (fullscreen())
     {
         if (_params.fullscreen_flip_left)
         {
-            std::swap(tex_coords[0][0][0], tex_coords[0][3][0]);
-            std::swap(tex_coords[0][0][1], tex_coords[0][3][1]);
-            std::swap(tex_coords[0][1][0], tex_coords[0][2][0]);
-            std::swap(tex_coords[0][1][1], tex_coords[0][2][1]);
+            std::swap(my_tex_coords[0][0][0], my_tex_coords[0][3][0]);
+            std::swap(my_tex_coords[0][0][1], my_tex_coords[0][3][1]);
+            std::swap(my_tex_coords[0][1][0], my_tex_coords[0][2][0]);
+            std::swap(my_tex_coords[0][1][1], my_tex_coords[0][2][1]);
         }
         if (_params.fullscreen_flop_left)
         {
-            std::swap(tex_coords[0][0][0], tex_coords[0][1][0]);
-            std::swap(tex_coords[0][0][1], tex_coords[0][1][1]);
-            std::swap(tex_coords[0][3][0], tex_coords[0][2][0]);
-            std::swap(tex_coords[0][3][1], tex_coords[0][2][1]);
+            std::swap(my_tex_coords[0][0][0], my_tex_coords[0][1][0]);
+            std::swap(my_tex_coords[0][0][1], my_tex_coords[0][1][1]);
+            std::swap(my_tex_coords[0][3][0], my_tex_coords[0][2][0]);
+            std::swap(my_tex_coords[0][3][1], my_tex_coords[0][2][1]);
         }
         if (_params.fullscreen_flip_right)
         {
-            std::swap(tex_coords[1][0][0], tex_coords[1][3][0]);
-            std::swap(tex_coords[1][0][1], tex_coords[1][3][1]);
-            std::swap(tex_coords[1][1][0], tex_coords[1][2][0]);
-            std::swap(tex_coords[1][1][1], tex_coords[1][2][1]);
+            std::swap(my_tex_coords[1][0][0], my_tex_coords[1][3][0]);
+            std::swap(my_tex_coords[1][0][1], my_tex_coords[1][3][1]);
+            std::swap(my_tex_coords[1][1][0], my_tex_coords[1][2][0]);
+            std::swap(my_tex_coords[1][1][1], my_tex_coords[1][2][1]);
         }
         if (_params.fullscreen_flop_right)
         {
-            std::swap(tex_coords[1][0][0], tex_coords[1][1][0]);
-            std::swap(tex_coords[1][0][1], tex_coords[1][1][1]);
-            std::swap(tex_coords[1][3][0], tex_coords[1][2][0]);
-            std::swap(tex_coords[1][3][1], tex_coords[1][2][1]);
+            std::swap(my_tex_coords[1][0][0], my_tex_coords[1][1][0]);
+            std::swap(my_tex_coords[1][0][1], my_tex_coords[1][1][1]);
+            std::swap(my_tex_coords[1][3][0], my_tex_coords[1][2][0]);
+            std::swap(my_tex_coords[1][3][1], my_tex_coords[1][2][1]);
         }
     }
 
@@ -952,11 +995,11 @@ void video_output::display_current_frame(
 
     glUseProgram(_render_prg);
     glActiveTexture(GL_TEXTURE0);
-    glBindTexture(GL_TEXTURE_2D, _color_srgb_tex[left]);
+    glBindTexture(GL_TEXTURE_2D, _color_tex[left]);
     if (left != right)
     {
         glActiveTexture(GL_TEXTURE1);
-        glBindTexture(GL_TEXTURE_2D, _color_srgb_tex[right]);
+        glBindTexture(GL_TEXTURE_2D, _color_tex[right]);
     }
     glActiveTexture(GL_TEXTURE2);
     glBindTexture(GL_TEXTURE_2D, (_input_subtitle_box[_active_index].is_valid()
@@ -999,10 +1042,10 @@ void video_output::display_current_frame(
     {
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 0.0f);
         glDrawBuffer(GL_BACK_LEFT);
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 1.0f);
         glDrawBuffer(GL_BACK_RIGHT);
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
     }
     else if (_params.stereo_mode == parameters::even_odd_rows
             || _params.stereo_mode == parameters::even_odd_columns
@@ -1016,7 +1059,7 @@ void video_output::display_current_frame(
         };
         glActiveTexture(GL_TEXTURE3);
         glBindTexture(GL_TEXTURE_2D, _render_mask_tex);
-        draw_quad(x, y, w, h, tex_coords, more_tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords, more_tex_coords);
     }
     else if (_params.stereo_mode == parameters::red_cyan_monochrome
             || _params.stereo_mode == parameters::red_cyan_half_color
@@ -1033,19 +1076,19 @@ void video_output::display_current_frame(
             || _params.stereo_mode == parameters::red_green_monochrome
             || _params.stereo_mode == parameters::red_blue_monochrome)
     {
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
     }
     else if (_params.stereo_mode == parameters::mono_left
             && !mono_right_instead_of_left)
     {
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 0.0f);
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
     }
     else if (_params.stereo_mode == parameters::mono_right
             || (_params.stereo_mode == parameters::mono_left && mono_right_instead_of_left))
     {
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 1.0f);
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
     }
     else if (_params.stereo_mode == parameters::left_right
             || _params.stereo_mode == parameters::left_right_half
@@ -1054,10 +1097,10 @@ void video_output::display_current_frame(
             || _params.stereo_mode == parameters::hdmi_frame_pack)
     {
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 0.0f);
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
         glViewport(viewport[1][0], viewport[1][1], viewport[1][2], viewport[1][3]);
         glUniform1f(glGetUniformLocation(_render_prg, "channel"), 1.0f);
-        draw_quad(x, y, w, h, tex_coords);
+        draw_quad(x, y, w, h, my_tex_coords);
     }
     assert(xgl::CheckError(HERE));
 }
@@ -1080,22 +1123,37 @@ void video_output::clear()
     assert(xgl::CheckError(HERE));
 }
 
-static void compute_viewport(int vp[4], int w, int h, float dst_w, float dst_h, float dst_ar, float src_ar)
+static void compute_viewport_and_tex_coords(int vp[4], float tc[4][2],
+        float src_ar, int w, int h, float dst_w, float dst_h, float dst_ar,
+        float zoom)
 {
-    vp[2] = dst_w;
-    vp[3] = dst_h;
     if (src_ar >= dst_ar)
     {
         // need black borders top and bottom
-        vp[3] = dst_ar / src_ar * dst_h;
+        float zoom_src_ar = zoom * dst_ar + (1.0f - zoom) * src_ar;
+        vp[2] = dst_w;
+        vp[3] = dst_ar / zoom_src_ar * dst_h;
+        vp[0] = (w - vp[2]) / 2;
+        vp[1] = (h - vp[3]) / 2;
+        float cutoff = (1.0f - zoom_src_ar / src_ar) / 2.0f;
+        tc[0][0] = cutoff;
+        tc[0][1] = 0.0f;
+        tc[1][0] = 1.0f - cutoff;
+        tc[1][1] = 0.0f;
+        tc[2][0] = 1.0f - cutoff;
+        tc[2][1] = 1.0f;
+        tc[3][0] = cutoff;
+        tc[3][1] = 1.0f;
     }
     else
     {
         // need black borders left and right
         vp[2] = src_ar / dst_ar * dst_w;
+        vp[3] = dst_h;
+        vp[0] = (w - vp[2]) / 2;
+        vp[1] = (h - vp[3]) / 2;
+        std::memcpy(tc, full_tex_coords[0], sizeof(tc));
     }
-    vp[0] = (w - vp[2]) / 2;
-    vp[1] = (h - vp[3]) / 2;
 }
 
 void video_output::reshape(int w, int h)
@@ -1110,6 +1168,7 @@ void video_output::reshape(int w, int h)
     _viewport[1][1] = 0;
     _viewport[1][2] = w;
     _viewport[1][3] = h;
+    std::memcpy(_tex_coords, full_tex_coords, sizeof(_tex_coords));
     glViewport(0, 0, w, h);
     clear();
     if (!_frame[_active_index].is_valid())
@@ -1129,11 +1188,11 @@ void video_output::reshape(int w, int h)
         {
             src_ar /= 2.0f;
         }
-        compute_viewport(_viewport[0], w / 2, h, dst_w, dst_h, dst_ar, src_ar);
+        compute_viewport_and_tex_coords(_viewport[0], _tex_coords[0], src_ar,
+                w / 2, h, dst_w, dst_h, dst_ar, _params.zoom);
+        std::memcpy(_viewport[1], _viewport[0], sizeof(_viewport[1]));
         _viewport[1][0] = _viewport[0][0] + w / 2;
-        _viewport[1][1] = _viewport[0][1];
-        _viewport[1][2] = _viewport[0][2];
-        _viewport[1][3] = _viewport[0][3];
+        std::memcpy(_tex_coords[1], _tex_coords[0], sizeof(_tex_coords[1]));
     }
     else if (_params.stereo_mode == parameters::top_bottom
             || _params.stereo_mode == parameters::top_bottom_half)
@@ -1146,11 +1205,11 @@ void video_output::reshape(int w, int h)
         {
             src_ar *= 2.0f;
         }
-        compute_viewport(_viewport[0], w, h / 2, dst_w, dst_h, dst_ar, src_ar);
-        _viewport[1][0] = _viewport[0][0];
+        compute_viewport_and_tex_coords(_viewport[0], _tex_coords[0], src_ar,
+                w, h / 2, dst_w, dst_h, dst_ar, _params.zoom);
+        std::memcpy(_viewport[1], _viewport[0], sizeof(_viewport[1]));
         _viewport[1][1] = _viewport[0][1] + h / 2;
-        _viewport[1][2] = _viewport[0][2];
-        _viewport[1][3] = _viewport[0][3];
+        std::memcpy(_tex_coords[1], _tex_coords[0], sizeof(_tex_coords[1]));
     }
     else if (_params.stereo_mode == parameters::hdmi_frame_pack)
     {
@@ -1166,11 +1225,11 @@ void video_output::reshape(int w, int h)
         float dst_h = (h - blank_lines) / 2;
         float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
         float src_ar = _frame[_active_index].aspect_ratio;
-        compute_viewport(_viewport[0], w, (h - blank_lines) / 2, dst_w, dst_h, dst_ar, src_ar);
-        _viewport[1][0] = _viewport[0][0];
+        compute_viewport_and_tex_coords(_viewport[0], _tex_coords[0], src_ar,
+                w, (h - blank_lines) / 2, dst_w, dst_h, dst_ar, _params.zoom);
+        std::memcpy(_viewport[1], _viewport[0], sizeof(_viewport[1]));
         _viewport[1][1] = _viewport[0][1] + (h - blank_lines) / 2 + blank_lines;
-        _viewport[1][2] = _viewport[0][2];
-        _viewport[1][3] = _viewport[0][3];
+        std::memcpy(_tex_coords[1], _tex_coords[0], sizeof(_tex_coords[1]));
     }
     else
     {
@@ -1178,11 +1237,10 @@ void video_output::reshape(int w, int h)
         float dst_h = h;
         float dst_ar = dst_w * screen_pixel_aspect_ratio() / dst_h;
         float src_ar = _frame[_active_index].aspect_ratio;
-        compute_viewport(_viewport[0], w, h, dst_w, dst_h, dst_ar, src_ar);
-        _viewport[1][0] = _viewport[0][0];
-        _viewport[1][1] = _viewport[0][1];
-        _viewport[1][2] = _viewport[0][2];
-        _viewport[1][3] = _viewport[0][3];
+        compute_viewport_and_tex_coords(_viewport[0], _tex_coords[0], src_ar,
+                w, h, dst_w, dst_h, dst_ar, _params.zoom);
+        std::memcpy(_viewport[1], _viewport[0], sizeof(_viewport[1]));
+        std::memcpy(_tex_coords[1], _tex_coords[0], sizeof(_tex_coords[1]));
     }
 }
 
