@@ -37,7 +37,6 @@ class video_output : public controller
 {
 private:
     bool _initialized;
-    bool _srgb_textures_are_broken;     // XXX: Hack: work around broken SRGB texture implementations
 
     /* We manage two frames, each with its own set of properties etc.
      * The active frame is the one that is displayed, the other frame is the one
@@ -66,14 +65,15 @@ private:
     video_frame _color_last_frame;      // last frame for this step; used for reinitialization check
     GLuint _color_prg;                  // color space transformation, color adjustment
     GLuint _color_fbo;                  // framebuffer object to render into the sRGB texture
-    GLuint _color_srgb_tex[2];          // output: sRGB texture
+    GLuint _color_tex[2];               // output: SRGB8 or linear RGB16 texture
     // Step 3: rendering
     parameters _render_last_params;     // last params for this step; used for reinitialization check
     GLuint _render_prg;                 // reads sRGB texture, renders according to _params[_active_index]
     GLuint _render_dummy_tex;           // an empty subtitle texture
     GLuint _render_mask_tex;            // for the masking modes even-odd-{rows,columns}, checkerboard
-    // OpenGL viewports for drawing the two views of the video frame
+    // OpenGL viewports and tex coordinates for drawing the two views of the video frame
     GLint _viewport[2][4];
+    float _tex_coords[2][4][2];
 
 private:
     // Step 1: initialize/deinitialize, and check if reinitialization is necessary
@@ -117,10 +117,11 @@ protected:
      * The second version is for everyone else.
      * TODO: This function needs to handle interlaced frames! */
     void display_current_frame(bool keep_viewport, bool mono_right_instead_of_left,
-            float x, float y, float w, float h, const GLint viewport[2][4]);
+            float x, float y, float w, float h,
+            const GLint viewport[2][4], const float tex_coords[2][4][2]);
     void display_current_frame()
     {
-        display_current_frame(false, false, -1.0f, -1.0f, 2.0f, 2.0f, _viewport);
+        display_current_frame(false, false, -1.0f, -1.0f, 2.0f, 2.0f, _viewport, _tex_coords);
     }
 
 public:
