@@ -1,7 +1,7 @@
 /*
  * This file is part of bino, a 3D video player.
  *
- * Copyright (C) 2010-2011
+ * Copyright (C) 2010, 2011, 2012
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <frederic.devernay@inrialpes.fr>
  * Joe <cuchac@email.cz>
@@ -130,6 +130,9 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
         }
         break;
     case Qt::Key_Q:
+#if QT_VERSION >= 0x040700
+    case Qt::Key_MediaStop:
+#endif
         _vo->send_cmd(command::toggle_play);
         break;
     case Qt::Key_E:
@@ -143,8 +146,23 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
         break;
     case Qt::Key_Space:
     case Qt::Key_P:
+#if QT_VERSION >= 0x040700
+    case Qt::Key_MediaTogglePlayPause:
+#endif
         _vo->send_cmd(command::toggle_pause);
         break;
+    case Qt::Key_MediaPlay:
+        if (_vo->_pausing)
+        {
+            _vo->send_cmd(command::toggle_pause);
+        }
+        break;
+    case Qt::Key_MediaPause:
+        if (!_vo->_pausing)
+        {
+            _vo->send_cmd(command::toggle_pause);
+        }
+        break;
     case Qt::Key_V:
         _vo->send_cmd(command::cycle_video_stream);
         break;
@@ -184,12 +202,38 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
     case Qt::Key_ParenRight:
         _vo->send_cmd(command::adjust_ghostbust, +0.01f);
         break;
-    case Qt::Key_Less:
+    case Qt::Key_BracketLeft:
         _vo->send_cmd(command::adjust_parallax, -0.01f);
         break;
-    case Qt::Key_Greater:
+    case Qt::Key_BracketRight:
         _vo->send_cmd(command::adjust_parallax, +0.01f);
         break;
+    case Qt::Key_Less:
+    case Qt::Key_ZoomOut:
+        _vo->send_cmd(command::adjust_zoom, -0.1f);
+        break;
+    case Qt::Key_Greater:
+    case Qt::Key_ZoomIn:
+        _vo->send_cmd(command::adjust_zoom, +0.1f);
+        break;
+    case Qt::Key_Slash:
+#if QT_VERSION >= 0x040700
+    case Qt::Key_VolumeDown:
+#endif
+        _vo->send_cmd(command::adjust_audio_volume, -0.05f);
+        break;
+    case Qt::Key_Asterisk:
+#if QT_VERSION >= 0x040700
+    case Qt::Key_VolumeUp:
+#endif
+        _vo->send_cmd(command::adjust_audio_volume, +0.05f);
+        break;
+    case Qt::Key_M:
+#if QT_VERSION >= 0x040700
+    case Qt::Key_VolumeMute:
+#endif
+        _vo->send_cmd(command::toggle_audio_mute);
+        break;
     case Qt::Key_Left:
         _vo->send_cmd(command::seek, -10.0f);
         break;
@@ -271,23 +315,21 @@ void video_container_widget::closeEvent(QCloseEvent *)
 
 /* The video_output_qt class */
 
-video_output_qt::video_output_qt(bool benchmark, video_container_widget *container_widget) :
+video_output_qt::video_output_qt(const int swap_interval, video_container_widget *container_widget) :
     video_output(),
     _container_widget(container_widget),
     _container_is_external(container_widget != NULL),
     _widget(NULL),
     _fullscreen(false),
-    _playing(false)
+    _playing(false),
+    _pausing(false)
 {
     if (!_container_widget)
     {
         _container_widget = new video_container_widget(NULL);
     }
     _format.setDoubleBuffer(true);
-    if (!benchmark)
-    {
-        _format.setSwapInterval(1);
-    }
+    _format.setSwapInterval(swap_interval);
     _format.setStereo(false);
 }
 
@@ -395,9 +437,7 @@ void video_output_qt::create_widget()
     _widget = new video_output_qt_widget(this, _format, _container_widget);
     if (!_widget->context()->isValid())
     {
-        // TODO: replace the error message with a better one once the freeze
-        // of translatable strings is over.
-        QMessageBox::critical(_widget, _("Error"), _("Cannot set OpenGL context format."));
+        QMessageBox::critical(_widget, _("Error"), _("Cannot get valid OpenGL context."));
         std::exit(1);
     }
     QObject::connect(_container_widget, SIGNAL(move_event()), _widget, SLOT(move_event()));
@@ -423,6 +463,10 @@ void video_output_qt::create_widget()
             std::exit(1);
         }
     }
+    msg::dbg("OpenGL framebuffer: %d:%d:%d bits for R:G:B",
+            _widget->format().redBufferSize(),
+            _widget->format().greenBufferSize(),
+            _widget->format().blueBufferSize());
     QGridLayout *container_layout = new QGridLayout();
     container_layout->addWidget(_widget, 0, 0);
     container_layout->setContentsMargins(0, 0, 0, 0);
@@ -741,6 +785,11 @@ void video_output_qt::receive_notification(const notification &note)
         std::istringstream current(note.current);
         s11n::load(current, _playing);
     }
+    else if (note.type == notification::pause)
+    {
+        std::istringstream current(note.current);
+        s11n::load(current, _pausing);
+    }
     /* More is currently not implemented.
      * In the future, an on-screen display might show hints about what happened. */
 }
