@@ -89,6 +89,7 @@ private:
     QGLFormat _format;
     bool _fullscreen;
     bool _playing;
+    bool _pausing;
 
     void create_widget();
     void mouse_set_pos(float dest);
@@ -109,7 +110,7 @@ public:
      * part of another widget (e.g. a main window). In this case, you also need
      * to use the move_event() function; see below. If no container widget is
      * given, we will use our own, and it will be a top-level window. */
-    video_output_qt(bool benchmark, video_container_widget *container_widget = NULL);
+    video_output_qt(const int swap_interval, video_container_widget *container_widget = NULL);
     virtual ~video_output_qt();
 
     /* If you give a container element to the constructor, you have to call
